<?php
/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 class CRM_FormProcessor_BAO_FormProcessorInstance extends CRM_FormProcessor_DAO_FormProcessorInstance {

   static $importingFormProcessors = array();

	/**
   * Function to get values
   *
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $formProcessorInstance->$key = $value;
        }
      }
    }
    $formProcessorInstance->find();
    while ($formProcessorInstance->fetch()) {
      $row = array();
      self::storeValues($formProcessorInstance, $row);
			$row['inputs'] = array_values(CRM_FormProcessor_BAO_FormProcessorInput::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			$row['actions'] = array_values(CRM_FormProcessor_BAO_FormProcessorAction::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			$row['default_data_inputs'] = array_values(CRM_FormProcessor_BAO_FormProcessorDefaultDataInput::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			$row['default_data_actions'] = array_values(CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));

			$handler = \Civi::service('form_processor_output_handler_factory')->getHandlerByName($row['output_handler']);
			if ($handler) {
				$configuration = $handler->getDefaultConfiguration();
				if (isset($row['output_handler_configuration']) && is_string($row['output_handler_configuration'])) {
					$row['output_handler_configuration'] = json_decode($row['output_handler_configuration'], true);
				}
				if (empty($row['output_handler_configuration']) || !is_array($row['output_handler_configuration'])) {
					$row['output_handler_configuration'] = array();
				}
				foreach($row['output_handler_configuration'] as $name => $value) {
					$configuration->set($name, $value);
				}
				$handler->setConfiguration($configuration);

				$row['output_handler'] = $handler;
			}

			if (isset($row['default_data_output_configuration']) && is_string($row['default_data_output_configuration'])) {
				$row['default_data_output_configuration'] = json_decode($row['default_data_output_configuration'], true);
			}
			if (empty($row['default_data_output_configuration']) || !is_array($row['default_data_output_configuration'])) {
				$row['default_data_output_configuration'] = array();
			}

      $result[$row['id']] = $row;
    }
    return $result;
  }

	/**
   * Function to add or update a FormProcessorInstance
   *
   * @param array $params
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a civirule rule');
    }

    if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorInstance', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorInstance', NULL, $params);
    }

    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $formProcessorInstance->$key = $value;
      }
    }
    if (!$formProcessorInstance->permission) {
      $formProcessorInstance->permission = '';
    }
    if (!isset($formProcessorInstance->name) || empty($formProcessorInstance->name)) {
      if (isset($formProcessorInstance->title)) {
        $formProcessorInstance->name = CRM_FormProcessor_BAO_FormProcessorInstance::buildNameFromLabel($formProcessorInstance->title);
      }
    }
		if (is_array($formProcessorInstance->output_handler_configuration)) {
			$formProcessorInstance->output_handler_configuration = json_encode($formProcessorInstance->output_handler_configuration);
		}
		if (is_array($formProcessorInstance->default_data_output_configuration)) {
			$formProcessorInstance->default_data_output_configuration = json_encode($formProcessorInstance->default_data_output_configuration);
		}

    $formProcessorInstance->save();
    self::storeValues($formProcessorInstance, $result);

    if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorInstance', $formProcessorInstance->id, $formProcessorInstance);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorInstance', $formProcessorInstance->id, $formProcessorInstance);
    }

    return $result;
  }

	/**
   * Function to delete a FormProcessorInstance with id
   *
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor');
    }

    CRM_Utils_Hook::pre('delete', 'FormProcessorInstance', $id, CRM_Core_DAO::$_nullArray);

		// First delete all inputs
		CRM_FormProcessor_BAO_FormProcessorInput::deleteWithFormProcessorInstanceId($id);
		CRM_FormProcessor_BAO_FormProcessorDefaultDataInput::deleteWithFormProcessorInstanceId($id);
		// And all actions
		CRM_FormProcessor_BAO_FormProcessorAction::deleteWithFormProcessorInstanceId($id);
		CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::deleteWithFormProcessorInstanceId($id);

    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    $formProcessorInstance->id = $id;
    $formProcessorInstance->delete();

    CRM_Utils_Hook::post('delete', 'FormProcessorInstance', $id, CRM_Core_DAO::$_nullArray);

    return;
  }

	/**
   * Public function to generate name from label
   *
   * @param $label
   * @return string
   * @access public
   * @static
   */
  public static function buildNameFromLabel($label) {
    $labelParts = explode(' ', strtolower($label));
    $nameString = implode('_', $labelParts);
    return substr($nameString, 0, 80);
  }

	/**
	 * Returns whether the name is valid or not
	 *
	 * @param string $name
	 * @param int $id optional
	 * @return bool
	 * @static
	 */
	public static function isNameValid($name, $id=null) {
		$invalidNames = array('getactions', 'getfields');
		if (in_array(strtolower($name), $invalidNames)) {
			return false;
		}
		$sql = "SELECT COUNT(*) FROM `civicrm_form_processor_instance` WHERE `name` = %1";
		$params[1] = array($name, 'String');
		if ($id) {
			$sql .= " AND `id` != %2";
			$params[2] = array($id, 'Integer');
		}
		$count = CRM_Core_DAO::singleValueQuery($sql, $params);
		return ($count > 0) ? false : true;
	}

  /**
   * Returns the permission for the form processor. When no permission is required
   * return an empty string.
   *
   * @param string $formProcessorName
   *   The name of the form processor.
   * @return string
   *   The required permission.
   */
  public static function getPermission($formProcessorName) {
    $sql = "SELECT `permission` FROM `civicrm_form_processor_instance` WHERE `name` = %1";
    $params[1] = array($formProcessorName, 'String');
    $permission = CRM_Core_DAO::singleValueQuery($sql, $params);
    return $permission;
  }

  /**
   * Returns the id of the form processor.
   *
   * @param string $formProcessorName
   *   The name of the form processor.
   * @return int
   *   The id of the form processor.
   */
  public static function getId($formProcessorName) {
    $sql = "SELECT `id` FROM `civicrm_form_processor_instance` WHERE `name` = %1";
    $params[1] = array($formProcessorName, 'String');
    $id = CRM_Core_DAO::singleValueQuery($sql, $params);
    return $id;
  }

  /**
   * Returns the status of the form processor.
   * @See CRM_FormProcessor_Status for possible values.
   *
   * @param string $formProcessorName
   *   The name of the form processor.
   * @return int
   *   The status of the form processor.
   */
  public static function getStatus($formProcessorName) {
    $sql = "SELECT `status` FROM `civicrm_form_processor_instance` WHERE `name` = %1";
    $params[1] = array($formProcessorName, 'String');
    $status = CRM_Core_DAO::singleValueQuery($sql, $params);
    return $status;
  }

  /**
   * Updates the status and source file of the form processor.
   * @See CRM_FormProcessor_Status for possible status values.
   *
   * @param string $formProcessorName
   *   The name of the form processor.
   * @param int $status
   *   The status value.
   * @param string $source_file
   *   The source file. Leave empty when status is IN_DATABASE.
   */
  public static function setStatusAndSourceFile($formProcessorName, $status, $source_file) {
    $sql = "UPDATE `civicrm_form_processor_instance` SET `status` = %2, `source_file` = %3 WHERE `name` = %1";
    $params[1] = array($formProcessorName, 'String');
    $params[2] = array($status, 'Integer');
    $params[3] = array($source_file, 'String');
    CRM_Core_DAO::executeQuery($sql, $params);
  }

   /**
    * Update the status from in code to overriden when a form processor has been changed
    *
    * @param $formProcessorId
    */
   public static function updateAndChekStatus($formProcessorId) {
     $sql = "SELECT `status`, `name` FROM `civicrm_form_processor_instance` WHERE `id` = %1";
     $params[1] = array($formProcessorId, 'Integer');
     $dao = CRM_Core_DAO::executeQuery($sql, $params);
     if ($dao->fetch()) {
       if (!in_array($dao->name, self::$importingFormProcessors) && $dao->status == CRM_FormProcessor_Status::IN_CODE) {
         $sql = "UPDATE `civicrm_form_processor_instance` SET `status` = %2 WHERE `id` = %1";
         $params[1] = array($formProcessorId, 'String');
         $params[2] = array(CRM_FormProcessor_Status::OVERRIDDEN, 'Integer');
         CRM_Core_DAO::executeQuery($sql, $params);
       }
     }
   }

   /**
    * Store the form processor name so we know that we are importing this form processor
    * and should not update its status on the way.
    *
    * @param $formProcessorName
    */
   public static function setFormProcessorToImportingState($formProcessorName) {
     self::$importingFormProcessors[] = $formProcessorName;
   }

 }
