<?php

namespace Civi\FormProcessor\Exporter;

class ExportToJson {
  
  /**
   * Exports a form processor to json.
   * 
   * @param int $form_processor_id
   *   The form processor.
   * @return string
   */
  public function export($form_processor_id) {
    $form_processor = civicrm_api3('FormProcessorInstance', 'getsingle', array('id' => $form_processor_id));
    // Strip all IDs
    unset($form_processor['id']);
    unset($form_processor['created_user_id']);
    unset($form_processor['created_date']);
    unset($form_processor['modified_user_id']);
    unset($form_processor['modified_date']);
    unset($form_processor['status']);
    unset($form_processor['source_file']);
    
    $form_processor['output_handler'] = $form_processor['output_handler']['name'];
    
    foreach($form_processor['inputs'] as $key => $input) {
      unset($form_processor['inputs'][$key]['id']);
      unset($form_processor['inputs'][$key]['form_processor_instance_id']);
      
      $form_processor['inputs'][$key]['type'] = $input['type']['name'];
      
      foreach($input['validators'] as $validator_key => $validator) {
        unset($form_processor['inputs'][$key]['validators'][$validator_key]['id']);
        unset($form_processor['inputs'][$key]['validators'][$validator_key]['entity']);
        unset($form_processor['inputs'][$key]['validators'][$validator_key]['entity_id']);  
        $form_processor['inputs'][$key]['validators'][$validator_key]['validator'] = $validator['validator']['name'];
      }
    }
    foreach($form_processor['actions'] as $key => $action) {
      unset($form_processor['actions'][$key]['id']);
      unset($form_processor['actions'][$key]['form_processor_instance_id']);
      unset($form_processor['actions'][$key]['weight']);
      if (isset($form_processor['actions'][$key]['condition_configuration']) && isset($form_processor['actions'][$key]['condition_configuration']['title'])) {
        unset($form_processor['actions'][$key]['condition_configuration']['title']);
      }
    }
    
    foreach($form_processor['default_data_inputs'] as $key => $input) {
      unset($form_processor['default_data_inputs'][$key]['id']);
      unset($form_processor['default_data_inputs'][$key]['form_processor_instance_id']);
      
      $form_processor['default_data_inputs'][$key]['type'] = $input['type']['name'];
      
      foreach($input['validators'] as $validator_key => $validator) {
        unset($form_processor['default_data_inputs'][$key]['validators'][$validator_key]['id']);
        unset($form_processor['default_data_inputs'][$key]['validators'][$validator_key]['entity']);
        unset($form_processor['default_data_inputs'][$key]['validators'][$validator_key]['entity_id']);  
        $form_processor['default_data_inputs'][$key]['validators'][$validator_key]['validator'] = $validator['validator']['name'];
      }
    }
    foreach($form_processor['default_data_actions'] as $key => $action) {
      unset($form_processor['default_data_actions'][$key]['id']);
      unset($form_processor['default_data_actions'][$key]['form_processor_instance_id']);
      unset($form_processor['default_data_actions'][$key]['weight']);
    }

    return $form_processor; 
  }

  public function import($data, $filename) {
    $form_processor_id = \CRM_FormProcessor_BAO_FormProcessorInstance::getId($data['name']);  
    $status = \CRM_FormProcessor_BAO_FormProcessorInstance::getStatus($data['name']);
    $new_status = null;
    $new_id = null;

    \CRM_FormProcessor_BAO_FormProcessorInstance::setFormProcessorToImportingState($data['name']);
    
    switch ($status) {
      case \CRM_FormProcessor_Status::IN_DATABASE:
        // Update to overriden
        \CRM_FormProcessor_BAO_FormProcessorInstance::setStatusAndSourceFile($data['name'], \CRM_FormProcessor_Status::OVERRIDDEN, $filename);
        $new_id = $form_processor_id;
        $new_status = \CRM_FormProcessor_Status::OVERRIDDEN;
        break;
      case \CRM_FormProcessor_Status::OVERRIDDEN:
        $new_id = $form_processor_id;
        $new_status = \CRM_FormProcessor_Status::OVERRIDDEN;
        break;
      default:
        $new_id = $this->importFormProcessor($data, $filename, $form_processor_id);
        $new_status = \CRM_FormProcessor_Status::IN_CODE;
        break;
    }

    $return = array(
      'original_id' => $form_processor_id,
      'new_id' => $new_id,
      'original_status' => $status,
      'new_status' => $new_status,
      'file' => $filename,
    );

    return $return;
  }
  
  private function importFormProcessor($data, $filename, $form_processor_id) {
    $params = $data;
    unset($params['inputs']);
    unset($params['actions']);
    unset($params['default_data_inputs']);
    unset($params['default_data_actions']);
    if ($form_processor_id) {
      $params['id'] = $form_processor_id;
    }
    $params['status'] = \CRM_FormProcessor_Status::IN_CODE;
    $params['source_file'] = $filename;
    $result = \CRM_FormProcessor_BAO_FormProcessorInstance::add($params);
    $id = $result['id'];
    
    // Clear all existing inputs and actions
    \CRM_FormProcessor_BAO_FormProcessorInput::deleteWithFormProcessorInstanceId($id);
    \CRM_FormProcessor_BAO_FormProcessorAction::deleteWithFormProcessorInstanceId($id);
    \CRM_FormProcessor_BAO_FormProcessorDefaultDataInput::deleteWithFormProcessorInstanceId($id);
    \CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::deleteWithFormProcessorInstanceId($id);
    
    foreach($data['inputs'] as $input) {
      $params = $input;
      unset($params['validators']);
      $params['form_processor_instance_id'] = $id;
      $result = \CRM_FormProcessor_BAO_FormProcessorInput::add($params);
      $input_id = $result['id'];
      foreach($input['validators'] as $validator) {
        $validator['entity'] = 'FormProcessorInput';
        $validator['entity_id'] = $input_id;
        \CRM_FormProcessor_BAO_FormProcessorValidation::add($validator);
      }
    }
    
    $weight = 1;
    foreach($data['actions'] as $action) {
      $params = $action;
      $params['form_processor_instance_id'] = $id;
      $params['weight'] = $weight;
      $result = \CRM_FormProcessor_BAO_FormProcessorAction::add($params);
      $weight ++;
    }
    
    foreach($data['default_data_inputs'] as $input) {
      $params = $input;
      unset($params['validators']);
      $params['form_processor_instance_id'] = $id;
      $result = \CRM_FormProcessor_BAO_FormProcessorDefaultDataInput::add($params);
      $input_id = $result['id'];
      foreach($input['validators'] as $validator) {
        $validator['entity'] = 'FormProcessorDefaultDataInput';
        $validator['entity_id'] = $input_id;
        \CRM_FormProcessor_BAO_FormProcessorValidation::add($validator);
      }
    }
    
    $weight = 1;
    foreach($data['default_data_actions'] as $action) {
      $params = $action;
      $params['form_processor_instance_id'] = $id;
      $params['weight'] = $weight;
      $result = \CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::add($params);
      $weight ++;
    }
    
    return $id;
  }

  /**
   * Imports form processor from files in an extension directory.
   * 
   * This scans the extension directory form-processors/ for json files.
   */
  public function importFromExtensions() {
    $return = array();
    $importedIds = array();
    $extensions = $this->getExtensionFileListWithFormProcessors();
    foreach($extensions as $ext_file) {
      $data = json_decode($ext_file['data'], true);
      $return[$ext_file['file']] = $this->import($data, $ext_file['file']);
      $importedIds[] = $return[$ext_file['file']]['new_id']; 
    }
    
    // Remove all form processors which are in code or overridden but not imported
    $dao = \CRM_Core_DAO::executeQuery("SELECT id, name FROM civicrm_form_processor_instance WHERE id NOT IN (".implode($importedIds, ",").") AND status IN (".\CRM_FormProcessor_Status::IN_CODE.", ".\CRM_FormProcessor_Status::OVERRIDDEN.")");
    while ($dao->fetch()) {
      \CRM_FormProcessor_BAO_FormProcessorInstance::deleteWithId($dao->id);
      $return['deleted form processors'][] = $dao->id.": ".$dao->name;
    }
    return $return;   
  }
  
  /**
   * Revert a form processor to the state in code.
   */
  public function revert($form_processor_id) {
    $dao = \CRM_Core_DAO::executeQuery("SELECT status, source_file FROM civicrm_form_processor_instance WHERE id = %1", array(1=>array($form_processor_id, 'Integer')));
    if (!$dao->fetch()) {
      return false;
    }
    if ($dao->status != \CRM_FormProcessor_Status::OVERRIDDEN) {
      return false;
    }
    $key = substr($dao->source_file, 0, stripos($dao->source_file, "/"));
    $extension = civicrm_api3('Extension', 'getsingle', array('key' => $key));
    $filename = $extension['path'].substr($dao->source_file, stripos($dao->source_file, "/"));
    $data = file_get_contents($filename);
    $data = json_decode($data, true);
    $this->importFormProcessor($data, $dao->source_file, $form_processor_id);
    return true;
  }
  
  /**
   * Returns a list with form-processor files within an extension folder.
   * 
   * @return array
   */
  private function getExtensionFileListWithFormProcessors() {
    $return = array();
    $extensions = civicrm_api3('Extension', 'get', array('options' => array('limit' => 0)));
    foreach($extensions['values'] as $ext) {
      if ($ext['status'] != 'installed') {
        continue;
      }
      
      $path = $ext['path'].'/form-processors';
      if (!is_dir($path)) {
        continue;
      }
      
      foreach (glob($path."/*.json") as $file) {
        $return[] = array(
          'file' => $ext['key']. '/form-processors/'.basename($file),
          'data' => file_get_contents($file),
        );  
      }
    }
    return $return;
  }
  
}
