<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\OutputHandler;

 use \Civi\FormProcessor\Config\ConfigurationBag;
 use \Civi\FormProcessor\Config\Specification;
 use \Civi\FormProcessor\Config\SpecificationBag;
 use \Civi\FormProcessor\OutputHandler\AbstractOutputHandler;
 use \Civi\FormProcessor\DataBag;

 use CRM_FormProcessor_ExtensionUtil as E;

 class OutputAllActionOutput extends AbstractOutputHandler {

	/**
	 * Get the configuration specification
	 *
	 * @return SpecificationBag
	 */
	public function getConfigurationSpecification() {
		return new SpecificationBag();
	}

	/**
	 * Convert the action data to output data.
	 *
	 * @param DataBag $dataBag
	 * @return array
	 */
	public function handle(DataBag $dataBag) {
		$output = array();
		foreach($dataBag->getAllInputs() as $input) {
			$output['input'][$input->name] = $dataBag->getInputData($input);
		}
		foreach($dataBag->getAllActions() as $action) {
			$actionData['action'] = $action->title;
			$actionData['output'] = $dataBag->getActionData($action);
			if (!is_array($actionData['output'])) {
				$actionData['output'] = array();
			}
			$output['action'][] = $actionData;
		}
		return $output;
	}

	/**
	 * Returns the label of the output handler.
	 *
	 * @return string
	 */
	public function getLabel() {
		return E::ts('Send everything');
	}

 }
