<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\Type;

 use \Civi\FormProcessor\Type\AbstractType;
 use \Civi\FormProcessor\Type\OptionListInterface;

 class CountryIsoCodeType extends AbstractType implements OptionListInterface {


  public function validateValue($value, $allValues=array()) {
    if (\CRM_Utils_Type::validate($value, 'String', false) === NULL) {
      return false;
    }

    $countries = $this->getOptions(array());
    if (!isset($countries[$value])) {
      return false;
    }

    return true;
  }

  /**
   * Returns the type number from CRM_Utils_Type
   */
  public function getCrmType() {
    return \CRM_Utils_Type::T_INT;
  }

  public function getOptions($params) {
    $countries = \CRM_Core_PseudoConstant::country();
    $iso_codes = \CRM_Core_PseudoConstant::countryIsoCode();
    $options = array();
    foreach($iso_codes as $id => $iso_code) {
      $options[$iso_code] = $countries[$id];
    }
    return $options;
  }

   /**
    * Normalize the input value.
    *
    * @param $value
    *
    * @return mixed
    */
   public function normalizeValue($value) {
     $iso_codes = \CRM_Core_PseudoConstant::countryIsoCode();
     foreach($iso_codes as $id => $iso_code) {
       if ($iso_code == $value) {
         return $id;
       }
     }
     return null;
   }

   /**
    * Denormalize the input value.
    *
    * @param $value
    *
    * @return mixed
    */
   public function denormalizeValue($value) {
     $iso_codes = \CRM_Core_PseudoConstant::countryIsoCode();
     if (isset($iso_codes[$value])) {
       return $iso_codes[$value];
     }
     return null;
   }

 }
