<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\Type;

 use Civi\FormProcessor\Config\ConfigurationBag;
 use Civi\FormProcessor\Config\Specification;
 use Civi\FormProcessor\Config\SpecificationBag;
 use Civi\FormProcessor\Config\SpecificationCollection;

 use \CRM_FormProcessor_ExtensionUtil as E;

 class CustomOptionListType extends AbstractType implements OptionListInterface {

   protected $normalizedOptions;

   protected $denormalizedOptions;

   protected $options;

   /**
    * Get the configuration specification
    *
    * @return SpecificationBag
    */
   public function getConfigurationSpecification() {
     $optionSpeBag = new SpecificationBag(array(
       new Specification('value', 'String', E::ts('Value'), true, null, null, null, null, ''),
       new Specification('label', 'String', E::ts('Label'), true,null, null, null, null, ''),
     ));

     return new SpecificationBag(array(
       new SpecificationCollection('options', E::ts('Options'), $optionSpeBag, 1, null)
     ));
   }


  public function validateValue($value, $allValues=array()) {
    $options = $this->getOptions($allValues);
    $type = 'String';
    if (!is_array($value)) {
      if (\CRM_Utils_Type::validate($value, $type, FALSE) === NULL) {
        return FALSE;
      }
      if (!isset($options[$value])) {
        return FALSE;
      }
    } else {
      return FALSE;
    }
    return true;
  }

  /**
   * Returns the type number from CRM_Utils_Type
   */
  public function getCrmType() {
    return \CRM_Utils_Type::T_STRING;
  }

   /**
    * Normalize the input value.
    *
    * @param $value
    *
    * @return mixed
    */
   public function normalizeValue($value) {
     $this->loadOptions();
     if (isset($this->normalizedOptions[$value])) {
       return $this->normalizedOptions[$value];
     } else {
       return null;
     }
   }

   /**
    * Denormalize the input value.
    *
    * @param $value
    *
    * @return mixed
    */
   public function denormalizeValue($value) {
     $this->loadOptions();
     if (isset($this->denormalizedOptions[$value])) {
       return $this->denormalizedOptions[$value];
     } else {
       return null;
     }
   }

  public function getOptions($params) {
    $this->loadOptions();
    return $this->options;
  }

  protected function loadOptions() {
    if ($this->options != null && $this->normalizedOptions != null && !$this->denormalizedOptions != null) {
      return;
    }

    $this->options = array();
    $this->normalizedOptions = array();
    $this->denormalizedOptions = array();
    foreach($this->configuration->get('options') as $option) {
      $this->options[$option['value']] = $option['label'];
      $this->normalizedOptions[$option['value']] = $option['value'];
      $this->denormalizedOptions[$option['value']] = $option['value'];
    }
  }

   public function setConfiguration(ConfigurationBag $configuration) {
    parent::setConfiguration($configuration);
    $this->normalizedOptions = null;
    $this->denormalizedOptions = null;
    $this->options = null;
    return $this;
   }

   /**
    * Sets the default values of this action
    */
   public function setDefaults() {
     parent::setDefaults();
     $this->normalizedOptions = null;
     $this->denormalizedOptions = null;
     $this->options = null;
   }

   public function toArray() {
     $return['name'] = $this->name;
     $return['label'] = $this->label;
     $return['default_value_description'] = $this->default_value_description;
     $return['configuration_spec'] = $this->getConfigurationSpecification()->toArray();
     $return['default_configuration'] = null;
     if ($this->getDefaultConfiguration()) {
       $return['default_configuration'] = $this->getDefaultConfiguration()->toArray();
     }
     return $return;
   }

 }
