<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

namespace Civi\FormProcessor\Type;

use Civi\FormProcessor\Config\Specification;
use Civi\FormProcessor\Config\SpecificationBag;
use Civi\FormProcessor\Type\AbstractType;
use Civi\FormProcessor\Type\OptionListInterface;

use CRM_FormProcessor_ExtensionUtil as E;

class MailingGroupType extends AbstractType implements OptionListInterface {

  /**
   * Get the configuration specification
   *
   * @return SpecificationBag
   */
  public function getConfigurationSpecification() {
    return new SpecificationBag([
      new Specification('multiple', 'Boolean', E::ts('Multiple'), FALSE, 0, NULL, [
        0 => E::ts('Single value'),
        1 => E::ts('Multiple values'),
      ]),
    ]);
  }

  public function validateValue($value, $allValues = []) {
    $multiple = $this->configuration->get('multiple') ? true : false;
    $groups = $this->getOptions();
    if ($multiple && is_array($value)) {
      foreach ($value as $valueItem) {
        if (\CRM_Utils_Type::validate($valueItem, 'Integer', FALSE) === NULL) {
          return FALSE;
        }
        if (!isset($groups[$valueItem])) {
          return FALSE;
        }
      }
    } elseif (!is_array($value) && $value) {
      if (\CRM_Utils_Type::validate($value, 'Integer', FALSE) === NULL) {
        return FALSE;
      }

      if (!isset($groups[$value])) {
        return FALSE;
      }
    } else {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Returns the type number from CRM_Utils_Type
   */
  public function getCrmType() {
    return \CRM_Utils_Type::T_INT;
  }

  /**
   * Normalize the input value.
   *
   * @param $value
   *
   * @return mixed
   */
  public function normalizeValue($value) {
    $multiple = $this->configuration->get('multiple') ? true : false;

    // Correct array values when field is not multiple.
    // this could be caused for example by a drupal webform with
    // a checkbox field. The value is submitted as an array, altough it contains
    // only one value.
    if (!$multiple && is_array($value) && count($value) == 1) {
      $value = reset($value);
    }

    return $value;
  }

  /**
   * Denormalize the input value.
   *
   * @param $value
   *
   * @return mixed
   */
  public function denormalizeValue($value) {
    $multiple = $this->configuration->get('multiple') ? true : false;
    if ($multiple && is_array($value)) {
      $value = [$value];
    }
    return $value;
  }

  public function getOptions($params) {
    static $return;
    if ($return) {
      return $return;
    }
    $groups = civicrm_api3('Group', 'get', [
      'is_active' => 1,
      'group_type' => 2,
      'options' => ['limit' => 0],
    ]);
    $return = [];
    foreach ($groups['values'] as $group) {
      $return[$group['id']] = $group['title'];
    }
    return $return;
  }

}
